<?php
/**
 * RegistroACAGED
 *
 * @package    Logic
 * @subpackage Registro/Caged
 *
 * @author     thalesvalias - 21/05/2012
 *
 */
class RegistroACAGED extends RegistroCAGED{

    const REGISTRO_A = 'A';
    const SEQUENCIA = '00001';
    const TIPO_LAYOUT = 'L2009';

    private $_estrutura;
    private $_nrInscricao;
    private $_tpInscricao;
    private $_totalEstabelecimento;
    private $_totalMovimentacao;
    private $_alteracao;

    /**
     * Gera Tipo de Registro A.
     *
     * @return void
     *
     * @author thalesvalias - 21/05/2012
     */
    public function geraRegistroA(){
        $this->_geraRegistro();
    }


    /**
     * Atribui os dados no atributo da classe Pai.
     *
     * @return void
     *
     * @author thalesvalias - 21/05/2012
     */
    private function _geraRegistro(){
        // Bloco 1 | 001 | Tipo do Registro.
        parent::geraRegistro(self::REGISTRO_A, null, null);

        // Bloco 2 | 002-006 | Tipo de Layout.
        $this->atribuiDado(self::TIPO_LAYOUT, 5);

        // Bloco 3 | 007-008 | Filler (Caracter branco).
        $this->atribuiDado(parent::CARACTER_BRANCO, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // @todo Passando 4 fixo por enquanto. Verificar como esta no Arte.
        // Bloco 2 | 009 | Meio Fisico (2-Disquete / 3-Fita / 4-Outros).
        $this->atribuiDado('4', 1);

        // Bloco 4 | 010-015 | Competencia (MMAAAA).
        $this->atribuiDado($this->getCompetencia(), 6);

        // Bloco 5 | 016 | Alteracao (1-Nada a alterar / 2-Alterar dados cadastrais).
        // Passando o 2 fixo, conforme ao Arte.
        $this->atribuiDado($this->_alteracao, 1);

        // Bloco 6 | 017-021 | Sequencia.
        $this->atribuiDado(self::SEQUENCIA, 5);

        // Bloco 7 | 022 | Tipo Identificador.
        $this->atribuiDado($this->_tpInscricao, 1);

        // Bloco 8 | 023-036 | Inscricao da Empresa.
        $this->atribuiDado($this->_nrInscricao, 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 9 | 037-071 | Razao Social.
        $this->atribuiDado($this->_estrutura->historico->getNmrazsocestrut(), 35, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 10 | 072-111 | Endereco.
        //atribui endereco
        $this->atribuiDado($this->_estrutura->endereco->getDsendereco(), 35, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        //atribui numero do endereco
        $this->atribuiDado($this->_estrutura->endereco->getNumero(), 5, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);

        // Bloco 11 | 112-119 | CEP.
        $this->atribuiDado($this->_estrutura->endereco->getNrcependereco(), 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 12 | 120-121 | UF.
        $this->atribuiDado($this->_estrutura->endereco->getSgestado(), 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 13 | 122-125 | DDD.
        $this->atribuiDado($this->_estrutura->endereco->ddd, 4, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 14 | 126-133 | Telefone.
        $this->atribuiDado($this->_estrutura->endereco->telefone, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 15 | 134-138 | Ramal.
        $this->atribuiDado($this->_estrutura->endereco->ramal, 5, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 16 | 139-143 | Total de Estabelecimentos.
        $this->atribuiDado((string)$this->_totalEstabelecimento, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 17 | 144-148 | Total de Movimentacoes.
        $this->atribuiDado((string)$this->_totalMovimentacao, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 18 | 149-240 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 92, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

    }


    public function setEstrutura($estrutura)
    {
        $this->_estrutura = $estrutura;
    }


    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }


    public function setSequencia($sequencia)
    {
        $this->_sequencia = $sequencia;
    }


    public function setTotalEstabelecimento($totalEstabelecimento)
    {
        $this->_totalEstabelecimento = $totalEstabelecimento;
    }


    public function setTotalMovimentacao($totalMovimentacao)
    {
        $this->_totalMovimentacao = $totalMovimentacao;
    }


    public function setAlteracao($alteracao)
    {
        $this->_alteracao = $alteracao;
    }


    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }


}