<?php
namespace Teknisa\Model\Geral\Repositories;

use HCM\Util\Repositories;

class Estruturam extends \Doctrine\ORM\EntityRepository {
    
    /**
     * Retorna todos os dados da estrutura (mestre e histórico)
     */
    public function getEstrutura($nrorg, $nrestruturam, $dtmescompetenc){
        
        $estruturam = Repositories::ESTRUTURAM;
        $estruturah = Repositories::ESTRUTURAH;
        
$sql = <<<SQL
SELECT estruturam
  FROM $estruturam estruturam
 WHERE estruturam.nrorg = :p_nrorg
   AND estruturam.nrestruturam = :p_nrestruturam
SQL;
        $params = array(
            'p_nrorg'          => $nrorg,
            'p_nrestruturam'   => $nrestruturam,
        );
        
        $estruturam = $this->getEntityManager()->createQuery($sql)->execute($params)[0];
        
        if (is_object($estruturam)) {
        
$sql = <<<SQL
SELECT estruturah
  FROM $estruturah estruturah
 WHERE estruturah.nrorg = :p_nrorg
   AND estruturah.nrestruturam = :p_nrestruturam
   AND estruturah.dtmescompetenc = (SELECT MAX(eh.dtmescompetenc)
                                      FROM $estruturah eh
                                     WHERE eh.nrorg = estruturah.nrorg
                                       AND eh.nrestruturam = estruturah.nrestruturam
                                       AND eh.dtmescompetenc <= :p_dtmescompetenc)
SQL;
            $params = array(
                'p_nrorg'          => $nrorg,
                'p_nrestruturam'   => $nrestruturam,
                'p_dtmescompetenc' => $dtmescompetenc,
            );
            
            $estruturah = $this->getEntityManager()->createQuery($sql)->execute($params)[0];
            
            if (is_object($estruturah)) {
                $estruturam->historico = $estruturah;
                return $estruturam;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
    
    public function getEstruturaPorCnpj($nrorg, $cdcnpjestrut, $dtmescompetenc){
        
        $estruturam = Repositories::ESTRUTURAM;
        $estruturah = Repositories::ESTRUTURAH;
        
$sql = <<<SQL
SELECT estruturah
  FROM $estruturah estruturah
 WHERE estruturah.nrorg = :p_nrorg
   AND estruturah.cdcnpjestrut = :p_cdcnpjestrut
   AND estruturah.dtmescompetenc = (SELECT MAX(eh.dtmescompetenc)
                                      FROM $estruturah eh
                                     WHERE eh.nrorg = estruturah.nrorg
                                       AND eh.nrestruturam = estruturah.nrestruturam
                                       AND eh.dtmescompetenc <= :p_dtmescompetenc)
SQL;
        $params = array(
            'p_nrorg'          => $nrorg,
            'p_cdcnpjestrut'   => $cdcnpjestrut,
            'p_dtmescompetenc' => $dtmescompetenc,
        );
        
        $estruturah = current($this->getEntityManager()->createQuery($sql)->execute($params));
        
        if (is_object($estruturah)) {
        
$sql = <<<SQL
SELECT estruturam
  FROM $estruturam estruturam
 WHERE estruturam.nrorg = :p_nrorg
   AND estruturam.nrestruturam = :p_nrestruturam
SQL;
            $params = array(
                'p_nrorg'          => $nrorg,
                'p_nrestruturam'   => $estruturah->getNrestruturam(),
            );
            
            $estruturam = current($this->getEntityManager()->createQuery($sql)->execute($params));
            
            if (is_object($estruturam)) {
                $estruturam->historico = $estruturah;
                return $estruturam;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
    
    public function retornaEstrutura($nrorg, $nrestruturam, $dtmescompetenc){
    
	$params = array('nrestruturam' => $nrestruturam, 'nrorg'=>$nrorg, 'dtmescompetenc' => $dtmescompetenc);
    $estruturam =  '\Teknisa\Model\Geral\Entities\Estruturam';

$sql = <<<SQL
SELECT s FROM $estruturam s
WHERE s.nrorg = :nrorg
    and s.nrestruturam = :nrestruturam    
    and s.dtinivigencia <= :dtmescompetenc
SQL;
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return $result[0];
    }
    
    public function getVincSalAbaixo5SM( $nrestruturam, $nrorg, $ano)
    {
        $ultdiano = '31/12/'.$ano;
        $primdiano = '01/01/'.$ano;
        $gpe_vinculom = Repositories::GPE_VINCULOM;
        $gpe_vinculoh = Repositories::GPE_VINCULOH;
        $gpe_altesalario = Repositories::GPE_ALTESALARIO;
        $fpa_valorreferenciah = Repositories::FPA_VALORREFERENCIAH;
        $sql = <<<SQL
    select 
        count(vm.nrvinculom) qtd
    from $gpe_vinculom vm
    inner join $gpe_vinculoh vh
        WITH vm.nrvinculom = vh.nrvinculom
        and vm.nrorg = vh.nrorg
    inner join $gpe_altesalario asl
        WITH vm.nrorg = asl.nrorg
        and vm.nrvinculom = asl.nrvinculom
    where 
            vm.nrtipovinculom in (1, 7, 9)
        and vm.nrorg = :nrorg
        and vm.dtadmissaovinc <= :ultdiano
        and vh.nrestrutlegal = :nrestruturam
        and (vm.dtrescisaovinc is null
            or vm.dtrescisaovinc >= :primdiano)
        and vh.dtmescompetenc = (select max(mvh.dtmescompetenc)
                                           from $gpe_vinculoh mvh
                                           where mvh.nrorg = vh.nrorg
                                              and mvh.nrvinculom = vh.nrvinculom
                                              and mvh.dtmescompetenc <= :ultdiano)
        and asl.dtaltesalario = ( select max(masl.dtaltesalario) dtaltesalario
                                              from $gpe_altesalario masl
                                              where masl.nrorg = asl.nrorg
                                                  and masl.nrvinculom = asl.nrvinculom
                                                  and masl.dtaltesalario <= :ultdiano)
        and asl.vrsalario < ( select (fh.vrreferencia*5)
                        from $fpa_valorreferenciah fh
                        where fh.nrvalorreferenciam = 1
                            and fh.dtmescompetenc = (select max(mfh.dtmescompetenc)
                                                from $fpa_valorreferenciah mfh
                                                where fh.nrvalorreferenciam = mfh.nrvalorreferenciam
                                                    and mfh.dtmescompetenc <= :ultdiano))
    group by vm.nrorg,
    vh.nrestrutlegal
SQL;

     
        $params = array(
            'ultdiano' => $ultdiano,
            'primdiano' => $primdiano,
            'nrestruturam' => $nrestruturam,
            'nrorg' => $nrorg
        
        );
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return ($result)?$result[0]:false;
    }
    
    public function getVincSalAcima5SM( $nrestruturam, $nrorg, $ano)
    {
        $ultdiano = '31/12/'.$ano;
        $primdiano = '01/01/'.$ano;
        $gpe_vinculom = Repositories::GPE_VINCULOM;
        $gpe_vinculoh = Repositories::GPE_VINCULOH;
        $gpe_altesalario = Repositories::GPE_ALTESALARIO;
        $fpa_valorreferenciah = Repositories::FPA_VALORREFERENCIAH;
        $sql = <<<SQL
    select 
        count(vm.nrvinculom) qtd
    from $gpe_vinculom vm
    inner join $gpe_vinculoh vh
        WITH vm.nrvinculom = vh.nrvinculom
        and vm.nrorg = vh.nrorg
    inner join $gpe_altesalario asl
        WITH vm.nrorg = asl.nrorg
        and vm.nrvinculom = asl.nrvinculom
    where 
            vm.nrtipovinculom in (1, 7, 9)
        and vm.nrorg = :nrorg
        and vm.dtadmissaovinc <= :ultdiano
        and vh.nrestrutlegal = :nrestruturam
        and (vm.dtrescisaovinc is null
            or vm.dtrescisaovinc >= :primdiano)
        and vh.dtmescompetenc = (select max(mvh.dtmescompetenc)
                                           from $gpe_vinculoh mvh
                                           where mvh.nrorg = vh.nrorg
                                              and mvh.nrvinculom = vh.nrvinculom
                                              and mvh.dtmescompetenc <= :ultdiano)
        and asl.dtaltesalario = ( select max(masl.dtaltesalario) dtaltesalario
                                              from $gpe_altesalario masl
                                              where masl.nrorg = asl.nrorg
                                                  and masl.nrvinculom = asl.nrvinculom
                                                  and masl.dtaltesalario <= :ultdiano)
        and asl.vrsalario > ( select (fh.vrreferencia*5)
                        from $fpa_valorreferenciah fh
                        where fh.nrvalorreferenciam = 1
                            and fh.dtmescompetenc = (select max(mfh.dtmescompetenc)
                                                from $fpa_valorreferenciah mfh
                                                where fh.nrvalorreferenciam = mfh.nrvalorreferenciam
                                                    and mfh.dtmescompetenc <= :ultdiano))
    group by vm.nrorg,
    vh.nrestrutlegal
SQL;

     
        $params = array(
            'ultdiano' => $ultdiano,
            'primdiano' => $primdiano,
            'nrestruturam' => $nrestruturam,
            'nrorg' => $nrorg
        
        );
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        return ($result)?$result[0]:false;
    }

    public function quantFuncPorPat( $nrestruturam, $nrorg, $nrtiposervpat, $ano)
    {
        $ultdiano = '31/12/'.$ano;
        $primdiano = '01/01/'.$ano;
        $gpe_vinculom = Repositories::GPE_VINCULOM;
        $gpe_vinculoh = Repositories::GPE_VINCULOH;
        $gpe_patvinculo = Repositories::GPE_PATVINCULO;
        try {
            $sql = <<<SQL
select count(distinct(pv.nrvinculom)) quant
from $gpe_patvinculo pv
inner join $gpe_vinculom vm
    with vm.nrvinculom = pv.nrvinculom
    and vm.nrorg = pv.nrorg
inner join $gpe_vinculoh vh
    with vm.nrvinculom = vh.nrvinculom
    and vm.nrorg = vh.nrorg
where pv.nrtiposervpat = :nrtiposervpat
    and vm.dtadmissaovinc <= :ultdiano
    and (vm.dtrescisaovinc is null
        or vm.dtrescisaovinc >= :primdiano)
    and vh.nrestrutlegal = :nrestruturam
    and vh.nrorg = :nrorg
    and vh.dtmescompetenc = (select max(mvh.dtmescompetenc)
                                     from $gpe_vinculoh mvh
                                     where mvh.nrvinculom = vh.nrvinculom
                                        and mvh.nrvinculom = vh.nrvinculom
                                        and mvh.dtmescompetenc <= :ultdiano)
SQL;
        $params = array(
            'ultdiano' => $ultdiano,
            'primdiano' => $primdiano,
            'nrestruturam' => $nrestruturam,
            'nrorg' => $nrorg,
            'nrtiposervpat' => $nrtiposervpat
        
        );
        $result = $this->getEntityManager()->createQuery($sql)->execute($params);
        $result = ($result)?$result[0]:false;
        return ($result)?$result['quant']:$result;

        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    public function getEstruturasRelacionadas($nrorg, $nrestruturamsup, $nrTipoHierarquia = null) {
        if (!empty($nrTipoHierarquia)) {
            $hierarqorg = $this->getEntityManager()->getRepository(Repositories::HIERARQUIAORG)->findBy(array('nrorg' => $nrorg,
            'nrestruturamsup' => $nrestruturamsup, 'nrtipohierarqorg' => $nrTipoHierarquia));
        } else {
            $hierarqorg = $this->getEntityManager()->getRepository(Repositories::HIERARQUIAORG)->findBy(array('nrorg' => $nrorg,
            'nrestruturamsup' => $nrestruturamsup));
        }

       
        if (empty($hierarqorg)) {
            return null;
        }

        $estruturasRelacionadas = array();
        foreach($hierarqorg as $estrutSup){
            $estruturaRelacionada = $this->getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findBy(array('nrorg' => $nrorg,
            'nrestruturam' => $estrutSup->getNrestruturamrel()));
            if (is_object($estruturaRelacionada[0])){
                $estruturasRelacionadas[] = $estruturaRelacionada;
            }
        }

        return $estruturasRelacionadas;
    }
    
    public function retornaEstruturaEducacional($nrorg, $dtmescompetenc, $cdcnpjestrut, $nmrazsocestrut){
        
        $estruturam = Repositories::ESTRUTURAM;
        $estruturah = Repositories::ESTRUTURAH;
        
$sql = <<<SQL
SELECT estruturam, estruturah
  FROM $estruturam estruturam
  JOIN $estruturah estruturah WITH estruturah.nrorg = estruturam.nrorg
                             AND estruturah.nrestruturam = estruturam.nrestruturam
                             AND estruturah.dtmescompetenc = (SELECT MAX(eh.dtmescompetenc)
                                                                FROM $estruturah eh
                                                               WHERE eh.nrorg = estruturah.nrorg
                                                                 AND eh.nrestruturam = estruturah.nrestruturam
                                                                 AND eh.dtmescompetenc <= :p_dtmescompetenc)
 WHERE (estruturam.nrtipoestrutura = 32 or estruturam.nrtipoestrutura = 14)
   AND estruturam.nrorg = :p_nrorg
   AND (estruturah.cdcnpjestrut = :p_cdcnpjestrut OR (:p_cdcnpjestrut IS NULL AND estruturah.nmrazsocestrut = :p_nmrazsocestrut)) 
SQL;
        $params = array(
            'p_nrorg'          => $nrorg,
            'p_dtmescompetenc' => $dtmescompetenc,
            'p_cdcnpjestrut'   => $cdcnpjestrut,
            'p_nmrazsocestrut' => $nmrazsocestrut
        );
        
        return $this->getEntityManager()->createQuery($sql)->execute($params);
    }
    
}